/**
 * Copyright (c) 2014-2018 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.registry_overlay.core;

import java.math.BigInteger;
import java.util.ArrayList;
import java.util.List;

import org.junit.Assert;
import org.ow2.petals.registry_overlay.api.Constants;
import org.ow2.petals.registry_overlay.configuration.generated.Credentials;
import org.ow2.petals.registry_overlay.configuration.generated.ManagementConsole;
import org.ow2.petals.registry_overlay.configuration.generated.Member;
import org.ow2.petals.registry_overlay.configuration.generated.Members;
import org.ow2.petals.registry_overlay.configuration.generated.PetalsRegistryOverlay;

import com.hazelcast.client.config.ClientConfig;
import com.hazelcast.client.config.ClientNetworkConfig;
import com.hazelcast.config.GroupConfig;

/**
 * A builder creating cluster configuration
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public class ClusterBuilder {

    protected final static String MEMBER_ID_1 = "memberId1";

    protected final static String MEMBER_ID_2 = "memberId2";

    protected final static String CREDENTIALS_GROUP = "my-petals-esb-registry";

    protected final static String CREDENTIALS_PWD = "my password of my petals esb registry";

    private ClusterBuilder() {
        // No constructor because utility class
    }

    /**
     * Create a very basic cluster configuration:
     * <ul>
     * <li>only one member (identifier: {@link #MEMBER_ID_1},</li>
     * <li>management console is enable on the default URL,</li>
     * <li>using the default TCP port</li>
     * <li>security is based on {@link #CREDENTIALS_GROUP} and {@link #CREDENTIALS_PWD}</li>
     * </ul>
     * 
     * @param port
     *            The port of the member
     * @return The very basic cluster configuration
     */
    protected static PetalsRegistryOverlay createVeryBasicClusterConfig(final int port) {
        final Member member = new Member();
        member.setId(MEMBER_ID_1);
        member.setValue("localhost");
        member.setPort(BigInteger.valueOf(port));
        final PetalsRegistryOverlay petalsRegistryOverlay = new PetalsRegistryOverlay();
        final Members members = new Members();
        petalsRegistryOverlay.setMembers(members);
        members.getMember().add(member);
        final Credentials credentials = new Credentials();
        credentials.setGroup(CREDENTIALS_GROUP);
        credentials.setPassword(CREDENTIALS_PWD);
        petalsRegistryOverlay.setCredentials(credentials);
        final ManagementConsole mngtConsole = new ManagementConsole();
        mngtConsole.setValue(RegistryOverlayNode.DEFAULT_MNGT_CENTER_URL);
        mngtConsole.setEnable(Boolean.TRUE);
        petalsRegistryOverlay.setManagementConsole(new ManagementConsole());

        return petalsRegistryOverlay;
    }

    /**
     * Same as {@link #createVeryBasicClusterConfig(int)} but with default port {@link Constants#DEFAULT_PORT}.
     * 
     * @return
     */
    protected static PetalsRegistryOverlay createVeryBasicClusterConfig() {
        return createVeryBasicClusterConfig(Constants.DEFAULT_PORT);
    }

    /**
     * Same as {@link #createVeryBasicClusterConfig(int)} but with a second member with id {@link #MEMBER_ID_2}.
     */
    protected static PetalsRegistryOverlay createVeryBasicClusterConfig(final int port, final int port2) {
        final PetalsRegistryOverlay config = createVeryBasicClusterConfig(port);

        Assert.assertNotEquals(config.getMembers().getMember().get(0).getPort(), port2);

        final Member member = new Member();
        member.setId(MEMBER_ID_2);
        member.setValue("localhost");
        member.setPort(BigInteger.valueOf(port2));

        config.getMembers().getMember().add(member);

        return config;
    }

    /**
     * Create the HZL client configuration to join a cluster created through
     * {@link #createVeryBasicClusterConfig(int)}
     * 
     * @param port
     *            The port on which the cluster is running
     * @return The HZL client configuration
     */
    protected static ClientConfig createClientConfigForVeryBasicCluster(final int port) {

        final ClientConfig hzlClientConfig = new ClientConfig();
        hzlClientConfig
                .setGroupConfig(new GroupConfig(ClusterBuilder.CREDENTIALS_GROUP, ClusterBuilder.CREDENTIALS_PWD));
        final ClientNetworkConfig networkConfig = new ClientNetworkConfig();
        final List<String> addresses = new ArrayList<String>();
        addresses.add("localhost:" + port);
        networkConfig.setAddresses(addresses);
        hzlClientConfig.setNetworkConfig(networkConfig);

        return hzlClientConfig;
    }

    /**
     * Same as {@link #createClientConfigForVeryBasicCluster(int)} but with default port {@link Constants#DEFAULT_PORT}.
     * 
     * @return
     */
    protected static ClientConfig createClientConfigForVeryBasicCluster() {
        return createClientConfigForVeryBasicCluster(Constants.DEFAULT_PORT);
    }

}
